var oscState;            //oscillation state flag to avoid setInterval() superposition

var parAct = [5.00, 0.80, 0.50];  //parameters actual values [rad/s, Hz, s]
var ampAct = 50;    //actual amplitude of the oscillating circle [%]
var steAct = [1.00, 0.1, 0.2];   //actual step size values [rad/s, Hz, s]
var numAct = 10;   //actual number of steps

var stepAll = 0.01                //step size of all parameters
var maxParams = [[0.1, 0.01, 0.06],  // rad/s, Hz, and s min values
                 [10.0, 1.60, 6.28]]  // rad/s, Hz, and s max values
var steParams = [0.01, 10];    //Step size parameters: [min, max]
var numParams = [1, 1000];      //Number of steps: [min, max]

var prec = 2;  //precision in showing floatting precison numbers
var unit = ['rad/s', 'Hz', 's'];  //measure units

var mIndex;
var actFreq;

var rullerAreas = '"oAr rAr sAr" "oAr rAr lAr"';
var noRullerAreas = '"oAr oAr sAr" "oAr oAr lAr"';

/*window.onload = () => {
  'use strict';

  if ('serviceWorker' in navigator) {
    navigator.serviceWorker
             .register('./sw.js');
  }
}*/

function updateMeasure() {
    
  mIndex = radioChecked('measure');
  
  var parFinal = parAct[mIndex] + steAct[mIndex]*numAct;  //angular frequency array final value [rad/s]
    
  document.getElementById("initValue").min = maxParams[0][mIndex];
  document.getElementById("initValue").max = maxParams[1][mIndex];  
  document.getElementById("initValue").value = parAct[mIndex];  
  document.getElementById("stepSize").value = steAct[mIndex];
  
  document.getElementById("initValueV").innerHTML = " " + unit[mIndex];
  document.getElementById("stepSizeV").innerHTML = " " + unit[mIndex];
  document.getElementById("finalValueV").innerHTML = parFinal.toFixed(prec).toString() + " " + unit[mIndex];
  
  updateParams();
}

function loadApp(){
  
  'use strict';

  if ('serviceWorker' in navigator) {
    navigator.serviceWorker
             .register('/sw.js');
  }

  rullerShow();  
  
  document.getElementById("initValue").step = stepAll;
  document.getElementById("stepSize").step = stepAll;
  document.getElementById("numSteps").step = 1;
  
  document.getElementById("stepSize").min = steParams[0];
  document.getElementById("stepSize").max = steParams[1];    
  
  document.getElementById("numSteps").min = numParams[0];
  document.getElementById("numSteps").max = numParams[1];  
  document.getElementById("numSteps").value = numAct;
  
  document.getElementById("ampOscillation").value = ampAct;
  
  updateMeasure();
          
  setOscillatorW();  
  
}

function setConfigW() {  
  document.getElementById("oscillatorW").style.display = "none";
  document.getElementById("configW").style.display = "grid";
  document.getElementById("aboutW").style.display = "none";
}

function setAboutW() {  
  document.getElementById("oscillatorW").style.display = "none";
  document.getElementById("configW").style.display = "none";
  document.getElementById("aboutW").style.display = "inline";
}

function setOscillatorW() {  
  document.getElementById("oscillatorW").style.display = "grid";
  document.getElementById("configW").style.display = "none";
  document.getElementById("aboutW").style.display = "none";

  var t0 = Date.now();
  var y, t;
  var cy = document.getElementById("oscCircle").cy.baseVal;  
  var y1 = document.getElementById("oscLine").y1.baseVal;
  var y2 = document.getElementById("oscLine").y2.baseVal;
  var amp = parseFloat(document.getElementById("ampOscillation").value);

  clearInterval(oscState);
  oscState = setInterval(oscillate, 20);

  function oscillate() {
    t = Date.now() - t0;
    y = 0.5*amp + 0.37*amp * Math.sin(actFreq * t * 1e-3);
    cy.valueAsString = y.toString() + "%";
    y1.valueAsString = y.toString() + "%";
    y2.valueAsString = y.toString() + "%";
  }
}

function radioChecked(radioName) {
  //Returns the index of the checked radio button
  //radioName: string with the name of the radio button
  var radio = document.getElementsByName(radioName);
  var i = 0;
  for (var option of radio) {    
    if (option.checked) return i;
    i++;
  }
}


function updateParams() {  
  
  var initValue = parseFloat(document.getElementById("initValue").value);  
  var stepSize = parseFloat(document.getElementById("stepSize").value);
  var numSteps = parseFloat(document.getElementById("numSteps").value);
  var amp = document.getElementById("ampOscillation").value;
    
  document.getElementById("ampOscillationV").innerHTML = amp + " %";
  document.getElementById("oscCircle").r.baseVal.valueAsString = amp/8 + "%";
  
  var finalValue = initValue + stepSize*numSteps;
  
  document.getElementById("finalValueV").innerHTML = finalValue.toFixed(prec).toString() + " " + unit[mIndex];
  
  document.getElementById("paramValue").min = initValue;
  document.getElementById("paramValue").max = finalValue;
  document.getElementById("paramValue").step = stepSize;
  document.getElementById("paramValue").value = initValue;
  
  updateOscillation();
  
}

function updateOscillation() {  
  var v = document.getElementById("paramValue").value;  
  
  if (mIndex == 0) actFreq = parseFloat(v);
  else if (mIndex == 1) actFreq = v*6.2832;
  else actFreq = 6.2832/v;  
    
  document.getElementById("paramValueV").innerHTML = parseFloat(v).toFixed(prec);
  document.getElementById("paramValueU").innerHTML = unit[mIndex];
}

function rullerShow() {
  if (document.getElementById("ruller").checked) {
    document.getElementById("oscillatorW").style.gridTemplateAreas = rullerAreas;
    document.getElementById("rullerC").style.display = "grid";    
  }
  else {
    document.getElementById("oscillatorW").style.gridTemplateAreas = noRullerAreas;
    document.getElementById("rullerC").style.display = "none";    
    //document.getElementById("rullerC").style.visibility = "hidden";
  }
  
}

function changeValue(st) {  
  var s = parseFloat(document.getElementById("paramValue").step);
  var v = parseFloat(document.getElementById("paramValue").value);
  if (st > 0) document.getElementById("paramValue").value = v+s;
  else document.getElementById("paramValue").value = v-s;
  updateOscillation();
}